"""Example component factory"""


def create_hello_world(graph):
    """
    Produces the string "hello world"

    Provided as demonstration and test bed for entrypoint-based registration.

    """
    return "hello world"
